/*
 * This file is part of nads
 * Copyright (c) 2003 ECSC Ltd.
 * Author: Gianni Tedesco <gianni@scaramanga.co.uk>
 * Released under the terms of the GNU GPL version 2
*/

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <nads.h>

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#ifdef HAVE_GETOPT_LONG
static const struct option opts[]={
	{"webserver",	1, 0, 'p'},
	{"help",	0, 0, 'h'},
	{NULL, 0, 0, 0}
};
#endif

/* usage:
 * @cmd: command name we were invoked with
 * @code: exit code
 *
 * Prints the usage string then exits.
 */
static __attribute__((noreturn)) void
usage(const char *cmd, int code)
{
	fprintf(stderr, "Usage: %s [-w webserver] < urls\n", cmd);
	fprintf(stderr, "N.A.D.S HTTP normalization test application\n\n");
	fprintf(stderr, "  --webserver, -w      Select default webserver emulation\n");
	fprintf(stderr, "  --help, -h           This message\n\n");
	exit(code);
}

int main(int argc, char **argv) 
{
	static char buf[8192];
	struct nads n = {.buf = buf, .host = NULL, .port = 0};
	const char *cmd = argv[0];
	const char *ws = NULL;
	int c;
	int state=0;
	int line;
	char *end;

#ifdef HAVE_GETOPT_LONG
	while( (c=getopt_long(argc, argv, "w:e:h?",
		opts, NULL)) != -1 )
#else
	while( (c=getopt(argc, argv, "w:e:h?")) != -1 )
#endif
	{
		switch ( c ) {
		case 'w':
			if ( ws )
				usage(cmd, 1);
			ws = optarg;
			break;
		case 'h':
			usage(cmd, 0);
			break;
		case '?':
			usage(cmd, 1);
			break;
		}
	}

	if ( ws && !nads_default_webserver(ws) ) {
		fprintf(stderr, "%s: %s: %s\n", argv[0], ws, nads_errstr());
		return 1;
	}

	for(line=1; fgets(buf, sizeof(buf), stdin); line++ ) {
		end = strchr(buf, '\n');
		if ( state == 1 ) {
			if ( end != NULL ) {
				fprintf(stderr, "%s: line %i too long\n",
					argv[0], line);
				state = 0;
			}
			continue;
		}else if ( end == NULL ) {
			state = 1;
			continue;
		}
		
		*end = '\0';
		if ( nads_normalize(&n) == NADS_OK ) {
			printf("%s?%s\n", n.uri, n.query);
		}else{
			fprintf(stderr, "%s: line %i: %s\n",
				argv[0], line, nads_errstr());
		}
	}

	return 0;
}
